local HOTF = ObjectiveTrackerFrame;
local Frame, Events = CreateFrame("Frame", "HideObjectiveTrackerFrame", ObjectiveTrackerFrame, BackdropTemplateMixin and "BackdropTemplate"), {};
local Debug = false;


-- send a message to chat frame
local function Display(msg)
  msg = msg or "";
  DEFAULT_CHAT_FRAME:AddMessage("\124cff20ffca[HideObjectiveTracker]\124r "..msg);
end

-- slash command toggle function.
local function HideObjectiveTracker_Toggle()
  if (HOTF:IsVisible() or HOTF:IsShown()) then
    HOTF:Hide();
    HideObjectiveTracker.hidden = true;
    -- Display("Hidden.");
  else
    HOTF:Show();
    HideObjectiveTracker.hidden = false;
    -- Display("Shown.");
  end
end

-- working macro creation function
local function HideObjectiveTracker_CreateMacro()
  -- create a macro if there is room and it doesn't already exist
  local totalMacros = {};
  -- nil for global, 1 for character
  local PerCharacter;
  totalMacros.Global, totalMacros.Char = GetNumMacros();
  -- this is checking if global macros is full and there is room in character macros
  -- then create it in character macros.  if global is not full, make it in global.
  -- otherwise set an error flag to abort.
  if (totalMacros.Global > 35 and totalMacros.Char < 18) then
    PerCharacter = 1;
  elseif (totalMacros.Global <= 35) then
    PerCharacter = nil;
  else
    PerCharacter = "ERROR";
  end
  -- macro hash to store macro variables
  local macro = {};
  macro.OriginalName = "Toggle Tracker";
  macro.OriginalTexture = "Achievement_quests_completed_06";
  macro.OriginalBody = "/tot";
  macro.Index = GetMacroIndexByName(macro.OriginalName);
  macro.Name, macro.Texture, macro.Body = GetMacroInfo(macro.Index);
  -- remove Interface\Icons from macro.Texture
  if (macro.Texture) then
    local Path1, Path2, Path3 = strsplit("\\", macro.Texture);
    macro.Texture = Path3;
  end
  -- if macros are full but the macro already exists lets check for that
  if (PerCharacter ~= "ERROR" or macro.OriginalTexture == macro.Texture) then
    -- if the macro does not currently exist, create it
    if (macro.Index == 0) then
      macro.Index = CreateMacro(macro.OriginalName, macro.OriginalTexture, macro.OriginalBody, PerCharacter);
	  Display("Macro 'Toggle Tracker' has been created.  Use /macro to access.");
    else
      -- preventative to try and make sure we aren't editing a different macro
	  if (macro.OriginalTexture == macro.Texture) then
        macro.Index = EditMacro(macro.Index, macro.OriginalName, macro.OriginalTexture, macro.OriginalBody);
        Display("Macro 'Toggle Tracker' has been recreated.  Use /macro to access.");
	  end
    end
  else
    Display("Error - Macros are full.  Access /macro to delete some.");
  end
end

-- slash command list.
SlashCmdList["HIDEOBJECTIVETRACKER"] = function(cmd)
  local cmd, args = strsplit(" ", cmd);
  if (cmd == "" or cmd == nil) then
    HideObjectiveTracker_Toggle();
  elseif (cmd == "macro") then
    HideObjectiveTracker_CreateMacro();
  elseif (cmd == "reset") then
    HideObjectiveTracker = {};
    HOTF:Show();
    HideObjectiveTracker.hidden = false;
  else
    Display("/tot - Toggle the Quest / Objective Tracker window.");
    Display("/tot macro - Create a /macro for toggling the window.");
    Display("/tot reset - Reset settings.");
  end    
end
SLASH_HIDEOBJECTIVETRACKER1 = "/tot";


-- main load
function Events:ADDON_LOADED(...)
  HOTF:SetMovable(true);
end

function Events:PLAYER_LOGIN(...)
  -- initialize db if required
  HideObjectiveTracker = HideObjectiveTracker or {};

  HOTF:SetMovable(true);
  HOTF:EnableMouse(true);
  HOTF:RegisterForDrag("LeftButton");
  HOTF:SetScript("OnDragStart", HOTF.StartMoving);
  HOTF:SetScript("OnDragStop", HOTF.StopMovingOrSizing);

  -- hide or show the objective tracker depending on last state saved setting
  -- also if the setting doesn't exist (first run), as we assume people installed this to hide the tracker
  if (HideObjectiveTracker.hidden == true) then
    HOTF:Hide();
  else
    HOTF:Show();
  end

  -- display addon loaded message
  Display("Loaded, use '/tot' to toggle, '/tot help' for more.");
end

Frame:SetScript("OnEvent", function(self, event, ...)
 Events[event](self, ...); -- call one of the functions above
end);

Frame:RegisterEvent("ADDON_LOADED");
Frame:RegisterEvent("PLAYER_LOGIN");

